
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>

#import "TickerWindow.h"

#define FEEDWINDOW_WIDTH   500
#define FEEDWINDOW_HEIGHT  500


@interface FeedWindow: NSWindow
{
    TickerWindow *ticker;

    NSFont *font;
    NSArray *articles;

    NSBrowser *headlines;
    NSTextView *content;

    NSColor *fg, *bg;
}

- initWithTickerWindow:(TickerWindow *)twin;

- setArticle:(id)sender;

- (int)browser:(NSBrowser *)sender numberOfRowsInColumn:(int)column;
- (void)browser:(NSBrowser *)sender willDisplayCell:(id)cell 
	  atRow:(int)row column:(int)column;


- (float)splitView:(NSSplitView *)sender 
constrainMinCoordinate:(float)proposedMin ofSubviewAt:(int)offset;
- (float)splitView:(NSSplitView *)sender 
constrainMaxCoordinate:(float)proposedMax ofSubviewAt:(int)offset;

- (void)tickerDidUpdate:(NSNotification *)aNotification;
- (void)tickerWindowWillClose:(NSNotification *)aNotification;

- articleText:(id)sender;
- articleBrowse:(id)sender;
- articleLink:(id)sender;
- articleShowAll:(id)sender;
- articleUpdate:(id)sender;

- (void)dealloc;

@end
