
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>

@interface Cache: NSObject
{
    NSMutableDictionary *data;
    NSMutableArray *queue;

    unsigned capacity;
}

- initWithCapacity:(unsigned)cap;

- (id)objectForKey:(id)aKey;
- (void)setObject:(id)anObject forKey:(id)aKey;

- (void)dealloc;

@end
