
#import <Foundation/NSString.h>
#import <Foundation/NSArray.h>

@interface NSString (StringAdditions)

- (NSString *)stringFromFirstLine;
- (NSArray *)stringToComponents;
- (NSString *)stringByTrimmingWhitespaceAndNewlines;
- (NSString *)firstComponent;
- (NSString *)stringByUnescapingHTML;

@end

