#! /bin/bash
#

if [ -r $1 ]; then
    cp $1 /tmp/ticker.html
    URL=/tmp/ticker.html
else
    URL=$1
fi

/usr/local/mozilla/mozilla -remote "openurl($URL,new-window)"

if [ $? -eq 0 ]; then
    echo contacted mozilla process
else
    echo starting mozilla
    /usr/local/mozilla/mozilla $URL &
fi

