
#import <AppKit/NSWindow.h>

#import "RSSHandler.h"
#import "TickerWindow.h"

#define TEXTWINDOW_WIDTH   500
#define TEXTWINDOW_HEIGHT  200

#define TEXTWINDOW_TITLE_FONT_INCREMENT 4

@interface TextWindow: NSWindow
{
    TickerWindow *ticker;
    int articleNumber;
}

- initWithTickerWindow:(TickerWindow *)twin 
	       article:(GenericArticle *)art 
		number:(int)val
	    attributes:(NSDictionary *)attr;

- (void)tickerDidUpdate:(NSNotification *)aNotification;
- (void)tickerWindowWillClose:(NSNotification *)aNotification;

- articleText:(id)sender;
- articleBrowse:(id)sender;
- articleLink:(id)sender;
- articleShowAll:(id)sender;
- articleUpdate:(id)sender;

- (void)dealloc;

@end
