
#import <Foundation/Foundation.h>

@interface MinMaxFormat : NSFormatter
{
    int minval, maxval;
}

- initWithMin:(int)min andMax:(int)max;

- (BOOL)getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString  **)error;
- (NSString *)stringForObjectValue:(id)anObject;

@end
