
#import <Foundation/Foundation.h>
#import <AppKit/NSMenu.h>

#import "Controller.h"
#import "HeadlineView.h"

@implementation HeadlineView

static NSMenu *_headline_menu = nil;

+ defaultMenu
{
    if(_headline_menu==nil){
	NSMenu *menu = 
	    [[NSMenu alloc] initWithTitle:_(@"Article")];
    
	[[menu addItemWithTitle: _(@"Text")
	       action: @selector (articleText:) 
	       keyEquivalent: @""] setTag:MENU_TEXT];
	[[menu addItemWithTitle: _(@"Browse")
	       action: @selector (articleBrowse:) 
	       keyEquivalent: @""] setTag:MENU_BROWSE];
	[[menu addItemWithTitle: _(@"Link")
	       action: @selector (articleLink:) 
	       keyEquivalent: @""] setTag:MENU_LINK];
	[[menu addItemWithTitle: _(@"Show all")
	       action: @selector (articleShowAll:) 
	       keyEquivalent: @""] setTag:MENU_SHOWALL];
	[[menu addItemWithTitle: _(@"Update")
	       action: @selector (articleUpdate:) 
	       keyEquivalent: @""] setTag:MENU_UPDATE];
	
	_headline_menu = menu;
    }

    return _headline_menu;
}

@end
