
#import <Foundation/NSScanner.h>

#import "DictionaryAdditions.h"

@implementation NSDictionary (DictionaryAdditions)

- (float)floatForKey:(NSString *)key
{
    float aFloat;
    [[NSScanner scannerWithString:[self objectForKey:key]]
	scanFloat:&aFloat];
    return aFloat;
}

@end
