
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>

@interface NSColor (ColorAsRGBADictionary)

+ (NSColor *)colorFromRGBADictionary:(NSDictionary *)dict; 
- (NSDictionary *)colorToRGBADictionary;

@end
